/*
Class responsible for evaluating the complexity measure of a question.
Returns a floating point value which is stored as the complexity measure.

Calculates complexity for the following question types:

Type 1 - Noun pluralization questions
Type 2 - Verb pluralization questions
Type 3 - Sentence pluralization questions
Type 4 - Scrambled sentence questions

Developed by Soham Hanuman Singh, sngsoh004@myuct.ac.za
*/

import java.io.FileNotFoundException;
import java.util.ArrayList;

public class DifficultyEvaluator 
{    
    public static double getDifficulty(Question q) throws FileNotFoundException //requests evaluation for a specific type of question and returns complexity measure value
    {
        switch (q.type)
        {
            case 1:
                return getNounDifficulty(q);
                
            case 2:
                return getVerbDifficulty(q,1);
                
            case 3:
                return getSentenceDifficulty(q);
                
            case 4:
                return getSentenceScrambleDifficulty(q);
        }   
        return 0;
    }
    
    private static double getSentenceScrambleDifficulty(Question q) //calculates complexity for scrambled sentences
    {
        return getSentenceLength(q) + getPOSDifficulty(q);
    }
    
    private static double getPOSDifficulty(Question q) //returns the number of different parts of speech in a sentence
    {
        ArrayList temp = new ArrayList();
        int count = 0;
        
        for (int i = 0; i < q.answerWords.length; i++) {
            if (!temp.contains(q.answerWords[i].type))
            {
                temp.add(q.answerWords[i].type);
                count++;
            }
        }
        return count;
    }
    
    private static double getSentenceLength(Question q) //returns the length of a sentence multiplied by 2
    {
        return q.answerWords.length*2;
    }
    
    private static double getNounDifficulty(Question q) //calculates complexity measure for noun pluralization questions
    {
        String prefix = getNounPrefix(q.sentence);
        return getNounSingularComplexity(prefix) + getNounOverlap(prefix);
    }
    
    private static double getVerbDifficulty(Question q, int ind) throws FileNotFoundException //requestions evaluation for a verb pluralization
    {
        String prefix = "", root = "", verb = "";
        
        if (ind == 1) //for verb pluralization questions, difficulty is dependant on noun in the sentence
        {
            prefix = getVerbPrefix(q.sentence);
            verb = findVerb(q.sentenceWords);
        }
        
        else if (ind == 2) //for sentence pluralization questions, difficulty is dependant on noun in the answer
        { 
            prefix = getVerbPrefix(q.answer);
            verb = findVerb(q.answerWords);
        }
        
        root = findRoot(verb);
        return getVerbSubjectConcordComplexity(prefix)  + getVerbRootComplexity(root);
    }
  
    private static String getNounPrefix(String text) //returns the prefix of a given noun
    {   
        if (text.toLowerCase().startsWith("umu")) return "Umu";
        
        else if (text.toLowerCase().startsWith("um")) return "Um";
        
        else if (text.toLowerCase().startsWith("ulu")) return "Ulu";
        
        else if (text.toLowerCase().startsWith("ulw")) return "Ulw";
        
        else if (text.toLowerCase().startsWith("u")) return "U";
        
        else if (text.toLowerCase().startsWith("ili")) return "Ili";
        
        else if (text.toLowerCase().startsWith("isi")) return "Isi";
        
        else if (text.toLowerCase().startsWith("is")) return "Is";
        
        else if (text.toLowerCase().startsWith("im")) return "Im";
        
        else if (text.toLowerCase().startsWith("in")) return "In";
        
        else if (text.toLowerCase().startsWith("i")) return "I";
        
        return "";
    }
    
    private static int getNounSingularComplexity(String prefix) //calculates complexity for a singular noun conversion to plural (without overlap)
    {        
        if (prefix.equalsIgnoreCase("Um") || prefix.equalsIgnoreCase("Umu")) return 4; //aba-, abe-, ab- + imi-
        
        else if (prefix.equalsIgnoreCase("Ulu") || prefix.equalsIgnoreCase("Ulw")) return 2; //izin-, izim-
        
        else if (prefix.equalsIgnoreCase("U")) return 3; //o-, izin-, izim-
        
        else if (prefix.equalsIgnoreCase("Is") || prefix.equalsIgnoreCase("Isi")) return 2; //iz-, izi-
        
        else if (prefix.equalsIgnoreCase("In") || prefix.equalsIgnoreCase("Im")) return 3; //izin-, izim-, ama-
        
        else if (prefix.equalsIgnoreCase("I") || prefix.equalsIgnoreCase("Ili")) return 2; //ama-, ame-
        
        return 0;
    }
    
    private static int getNounOverlap(String prefix) //calculates complexity of overlap for a singular noun conversion to plural
    {   
        if (prefix.equalsIgnoreCase("Um") || prefix.equalsIgnoreCase("Umu")) return 1; //aba,abe,ab + imi
        
        else if (prefix.equalsIgnoreCase("Ulu") || prefix.equalsIgnoreCase("Ulw")) return 0; //no overlap
        
        else if (prefix.equalsIgnoreCase("U")) return 1; //o + izin,izim
        
        else if (prefix.equalsIgnoreCase("Is") || prefix.equalsIgnoreCase("Isi")) return 0; //no overlap
        
        else if (prefix.equalsIgnoreCase("In") || prefix.equalsIgnoreCase("Im")) return 1; //izin,izim + ama
        
        else if (prefix.equalsIgnoreCase("I") || prefix.equalsIgnoreCase("Ili")) return 0; //no overlap
              
        return 0;
    }
    
    private static String getVerbPrefix(String text) //returns the prefix of a given verb
    {
        if (text.toLowerCase().startsWith("aba")) return "Aba"; 
        
        else if (text.toLowerCase().startsWith("abe")) return "Abe"; 
        
        else if (text.toLowerCase().startsWith("ab")) return "Ab";
        
        else if (text.toLowerCase().startsWith("ama")) return "Ama"; 
        
        else if (text.toLowerCase().startsWith("ame")) return "Ame"; 
        
        else if (text.toLowerCase().startsWith("izim")) return "Izim"; 
        
        else if (text.toLowerCase().startsWith("izin")) return "Izin"; 
        
        else if (text.toLowerCase().startsWith("izi")) return "Izi"; 
        
        else if (text.toLowerCase().startsWith("imi")) return "Imi"; 
        
        else if (text.toLowerCase().startsWith("iz")) return "Iz"; 
        
        else if (text.toLowerCase().startsWith("o")) return "O"; 
        
        return "";
    }
    
    private static int getVerbSubjectConcordComplexity(String prefix) //calculates the complexity of a verb pluralization based on a given pluralized noun prefix
    {
        if (prefix.equalsIgnoreCase("Aba") || prefix.equalsIgnoreCase("Abe") 
                || prefix.equals("Ab") 
                || prefix.equalsIgnoreCase("O")) return 4; //all map to ba-
        
        else if (prefix.equalsIgnoreCase("Imi")) return 1; //maps to i-
        
        else if (prefix.equalsIgnoreCase("Ama") || prefix.equalsIgnoreCase("Ame")) return 2; //both map to a-
        
        else if (prefix.equalsIgnoreCase("Iz") || prefix.equalsIgnoreCase("Izi")
                || prefix.equals("Izin") 
                || prefix.equalsIgnoreCase("Izim")) return 4; //all map to zi-
        
        return 0;
    }
    
    private static String findRoot(String text) throws FileNotFoundException //finds the root of a given verb using verb root list in Tester
    {
        int maxLength = 0;
        String root = "";
        for (int i = 0; i < Tester.roots.size(); i++) {
            if (text.toLowerCase().contains(Tester.roots.get(i).toLowerCase()) && Tester.roots.get(i).length()>maxLength)
            {
                maxLength = Tester.roots.get(i).length();
                root = Tester.roots.get(i);
            }
        }
        return root;
    }
    
    private static int getVerbRootComplexity(String root) //calculates added complexity if verb root begins with a vowel
    {
        if (root.toLowerCase().startsWith("a") || root.toLowerCase().startsWith("e")
                || root.toLowerCase().startsWith("i") || root.toLowerCase().startsWith("o")
                || root.toLowerCase().startsWith("u")) return 1;
        
        else return 0;
    }
    
    private static String findVerb(Word [] sentence) //returns the verb in a given sentence
    {
        for (int i = 0; i < sentence.length; i++) {
            if (sentence[i].type.contains("v")) return sentence[i].text;
        }
        return "";
    }

    private static double getSentenceDifficulty(Question q) throws FileNotFoundException { //calculates the difficulty of a sentence pluralization question
        return getNounDifficulty(q) + getVerbDifficulty(q,2);
    }
    
}
